<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:saxon="http://icl.com/saxon" exclude-result-prefixes="saxon">
	<xsl:output method="xhtml" indent="yes"/>
	<xsl:param name="CAM_Template"/>
	<xsl:param name="XML_File"/>	
	<xsl:variable name="version">1.05</xsl:variable>
	<!-- Modified December, 2009 for CAMV ERRORCODE, CAMINFO, CAMWARN handling -->
	<xsl:template match="/">
		<html>
			<head>
				<meta content="text/html; charset=UTF-8" http-equiv="Content-Type"/>
				<title>CAM Exchange Validation Results Report</title>
			</head>
			
			<body>

				<p><img src="..\cam.jpg" alt="Logo"/></p>

				<p><br/></p>
				
				<table border="0" width="100%">
					<tr>
						<td>
							<b>Validation Results Report</b>
						</td>
						<td>
							<xsl:value-of select="$CAM_Template"/>
						</td>
						<td>Version: <xsl:value-of select="$version"/></td>
					</tr>
				</table>

				<p><br/></p>
				<xsl:variable name="countErrors">
					<xsl:for-each select="//node()/@*/name()">
						<xsl:if test="contains(.,'CAMERROR') or(contains(.,'CAMWARN')) or(contains(.,'CAMINFO'))"><error/></xsl:if>
					</xsl:for-each>
				</xsl:variable>
				
				<xsl:choose>
				<xsl:when test="count($countErrors//error) &gt; 0">
				<table border="0" width="100%" cellpadding="10">
					<tr  style="background-color:yellow">
						<th style="width:10%;height:35">
							
						</th>
						<th style="width:50%;height:35;text-align:left">
							<strong style="font:arial">Validation Detail</strong>
						</th>
						<th style="width:25%;height:35;text-align:left">
							<strong style="font:arial">Location</strong>
						</th>
						
					</tr>

					<!-- Now write the table rows out just here -->
					<xsl:apply-templates/>
					<!-- Now write the table last row at end -->
					<tr style="background-color:yellow">
						<th style="width:10%;height:35"></th>
						<th style="width:50%;height:35"></th>
						<th style="width:25%;height:35"></th>
					</tr>
				</table>
				</xsl:when>
				<xsl:otherwise>
					<table border="0" width="100%">
						<tr>
							<td>
								<b>Validation reported no problems</b>
							</td>
							<td>
							
							</td>
						</tr>
					</table>
				</xsl:otherwise>
				</xsl:choose>
			</body>
		</html>
	</xsl:template>

	<xsl:template match="//*">
		<xsl:variable name="errors">
			<xsl:for-each select="@*">
				<xsl:variable name="chkName" select="name()"/>
				<xsl:if test="contains($chkName,'CAMERROR')"><error name="{$chkName}"><xsl:value-of select="."/></error></xsl:if>
				<xsl:if test="contains($chkName,'CAMWARN')"><error name="{$chkName}">WARNING: <xsl:value-of select="."/></error></xsl:if>
				<xsl:if test="contains($chkName,'CAMINFO')"><error name="{$chkName}">INFO: <xsl:value-of select="."/></error></xsl:if>
			</xsl:for-each>
		</xsl:variable>
		<xsl:if test="count($errors//error) &gt; 0">
		<xsl:variable name="thisPath"><xsl:call-template name="path"/></xsl:variable>
		
		<xsl:variable name="errorDetails">
			<xsl:for-each select="@*">
				<xsl:variable name="chkName" select="name()"/>
				<xsl:if test="contains($chkName,'CAMERROR') and(not(contains($chkName,'CAMERRORCODE')))"><error name="{$chkName}"><xsl:value-of select="."/></error></xsl:if>
				<xsl:if test="contains($chkName,'CAMWARN')"><error name="{$chkName}">WARNING: <xsl:value-of select="."/></error></xsl:if>
				<xsl:if test="contains($chkName,'CAMINFO')"><error name="{$chkName}">INFO: <xsl:value-of select="."/></error></xsl:if>
			</xsl:for-each>
		</xsl:variable>
		<xsl:for-each select="$errors//error">
			<xsl:variable name="this" select="position()"/>
			<xsl:variable name="fullPath">
				<xsl:value-of select="$thisPath"/>
				<!-- xsl:choose>
					<xsl:when test="@name='CAMERRORCODE'"><xsl:value-of select="$thisPath"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="concat($thisPath,'/@',substring-after(@name,'CAMERRORCODE'))"/></xsl:otherwise>
				</xsl:choose -->
			</xsl:variable>
			<xsl:choose>
				<xsl:when test="string-length(substring-after(.,';')) =0 ">
					<xsl:call-template name="printError">
						<xsl:with-param name="code" select="."/>
						<xsl:with-param name="error" select="$errorDetails/error[$this]"/>
						<xsl:with-param name="path" select="$fullPath"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="splitError">
						<xsl:with-param name="code" select="."/>
						<xsl:with-param name="error" select="$errorDetails/error[$this]"/>
						<xsl:with-param name="path" select="$fullPath"/>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
		</xsl:if>
		<xsl:apply-templates select="child::*"/>
	</xsl:template>
	<xsl:template name="splitError">
		<xsl:param name="code"/>
		<xsl:param name="error"/>
		<xsl:param name="path"/>
		<xsl:if test="string-length($code) &gt; 0">
			<xsl:call-template name="printError">
				<xsl:with-param name="code" select="substring-before($code,';')"/>
				<xsl:with-param name="error" select="substring-before($error,';')"/>
				<xsl:with-param name="path" select="$path"/>
			</xsl:call-template>

			<xsl:call-template name="splitError">
				<xsl:with-param name="code" select="substring-after($code,';')"/>
				<xsl:with-param name="error" select="substring-after($error,';')"/>
				<xsl:with-param name="path" select="$path"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<xsl:template match="*|@*|text()|node()">
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template name="path">
		<xsl:variable name="name" select="name()"/>
		<xsl:for-each select="parent::*">
			<xsl:call-template name="path"/>
		</xsl:for-each>

		<xsl:value-of select="concat('/',name(),'[',count(preceding-sibling::*[name()=$name])+1,']')"/>
	</xsl:template>

	<xsl:template name="printError">
		<xsl:param name="code"/>
		<xsl:param name="error"/>
		<xsl:param name="path"/>
		<tr>
			<td style="width:10%;font-size:12"> </td>
			<!-- td style="width:20%;font-size:12">
					<xsl:value-of select="$code"/>
			</td -->
			<td style="width:50%;font-size:12">
					<xsl:value-of select="$error"/>
			</td>
			<td style="width:25%;font-size:12">
					<xsl:value-of select="$path"/>
			</td>
		</tr>
	</xsl:template>
</xsl:stylesheet>
