<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:w2e="http://www.oasis-open.org/cam/functions"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:camed="http://jcam.org.uk/editor"
    xmlns:cam2func="http://jcam.org.uk/editor/function"
    exclude-result-prefixes="xsd as w2e cam2func">
    <!-- CAM kit utility program -->
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
        Authors: David RR Webber, September, 2012 -->
    <!--
        Purpose  : Parse SQL table details - produce XML components for drag and drop insertion into XML structure in CAM template.
        Original : 09/09/2012
        Initials : drrw
        CAM ver    : 1.1 
        Description : Part of a set of tools to integrate with SQL data sources and CAM.
        Updated  - Inits - Change
        09/09/12   drrw   Initial release
        09/17/12   drrw   Fix for DBname catalog or schema
        09/17/12   drrw   Make setLength from (1-{length})
        10/03/12   drrw   Add version number message
        11/01/12   drrw   Resolve SQL INT rule mapping
        11/01/12   drrw   Improve parent element naming details
        03/02/13   drrw   Fix QName error with lowercase names - just output the input if no rules matched
        -->

    <xsl:param name="prefixXMLns"/>
    <xsl:param name="prefixXMLurl"/>
    <xsl:param name="renameMode">NIEM</xsl:param><!-- NIEM or camel case -->

    <xsl:variable name="version">1.05</xsl:variable>
    <xsl:variable name="numbers">###########################################################################################################################################################################################################################################################################</xsl:variable>
    <xsl:variable name="SQLdefinitions" select="/"/>
    
    <xsl:variable name="nsTrue" as="xsd:boolean" select="if(string-length($prefixXMLns) &gt; 0) then true() else false()"/>
    <xsl:variable name="nsPrefix"><xsl:if test="$nsTrue"><xsl:value-of select="concat($prefixXMLns,':')"/></xsl:if></xsl:variable>
    
    <xsl:variable name="rules" select="/"/> 
    <xsl:variable name="source" select="/"/> 
    <xsl:variable name="ignoreDomainsNDR" select="if($renameMode='NIEM') then 'false' else 'true'"/>
    
    <xsl:include href="../word-utils.xsl"/>
    
    <xsl:include href="../cam-NDR-functions.xsl"/>

    <xsl:output method="xml" indent="yes" exclude-result-prefixes="w2e"/>

    <xsl:template match="/">
        <xsl:message>Processing SQL table components - version <xsl:value-of select="$version"/></xsl:message>
        <xsl:message> - mode <xsl:value-of select="$renameMode"/></xsl:message>
            <xsl:call-template name="generateDictionary"/>
        <xsl:message>SQL Table components loader completed</xsl:message>
    </xsl:template>

    <xsl:template name="generateDictionary">
        <xsl:element name="as:CAM">
            <xsl:namespace name="as">http://www.oasis-open.org/committees/cam</xsl:namespace>
            <xsl:namespace name="camed">http://jcam.org.uk/editor</xsl:namespace>
            <xsl:if test="string-length($prefixXMLns) &gt; 0">
                <xsl:namespace name="{$prefixXMLns}"><xsl:value-of select="$prefixXMLurl"/></xsl:namespace>
            </xsl:if>
            <xsl:attribute name="CAMlevel">1</xsl:attribute>
            <xsl:attribute name="version">1.0</xsl:attribute>
            <as:Header>
                <as:Description>Generated by CAM-SQL-designer.xsl</as:Description>
                <as:Owner>SQL Designer tool</as:Owner>
                <as:Version>0.1</as:Version>
                <as:DateTime><xsl:value-of select="current-dateTime()"/></as:DateTime>
            </as:Header>
            <xsl:element name="as:AssemblyStructure">
              <xsl:element name="as:Structure">
                <xsl:attribute name="ID" select="'SQLimport'"/>
                <xsl:attribute name="reference"/>
                <xsl:attribute name="taxonomy">XML</xsl:attribute>
                <!-- xsl:element name="as:Namespaces">
                    <xsl:if test="string-length($prefixXMLns) &gt; 0">
                        <xsl:element name="as:namespace">
                            <xsl:attribute name="prefix"><xsl:value-of select="$prefixXMLns"/></xsl:attribute>
                            <xsl:value-of select="$prefixXMLurl"/>
                        </xsl:element>
                    </xsl:if>
                </xsl:element -->

                <xsl:for-each select="$SQLdefinitions//TableEntity">
                    <xsl:variable name="currentName" select="@Name"/>
                    <xsl:message> - processing table - <xsl:value-of select="$currentName"/> <xsl:if test="$nsTrue"> - namespace <xsl:value-of select="$prefixXMLns"/></xsl:if> </xsl:message>
                    <xsl:variable name="shortname" select="@shortname"/>
                    <!-- xsl:variable name="thisName"><xsl:value-of select="concat($nsPrefix,$currentName)"/>details</xsl:variable -->
                    
                    <xsl:variable name="thisName">
                        <xsl:call-template name="renameSQL">
                            <xsl:with-param name="SQLname" select="$currentName"/>
                            <xsl:with-param name="typeName" select="string"/>
                            <xsl:with-param name="ns" select="if(string-length($prefixXMLns) &gt; 0) then concat($prefixXMLns,':') else ''"/>
                        </xsl:call-template>
                    </xsl:variable>
                    
                    <!-- xsl:message> - <xsl:value-of select="$thisName"/></xsl:message -->
                        <xsl:element name="{$thisName}" namespace="{$prefixXMLurl}">
                            <xsl:if test="string-length($prefixXMLns) &gt; 0">
                                <xsl:namespace name="{$prefixXMLns}"><xsl:value-of select="$prefixXMLurl"/></xsl:namespace>
                            </xsl:if>
                            <!-- xsl:attribute name="UID"><xsl:value-of select="if(string-length($prefixXMLns) &gt; 0) then $prefixXMLns else 'cam'"/>-<xsl:value-of select="generate-id()"/></xsl:attribute -->
                            
                            <xsl:for-each select="$SQLdefinitions//TableEntity/Column">
                                <!-- xsl:variable name="childName" select="@Name"/ -->
                                
                                <xsl:variable name="childName">
                                    <xsl:call-template name="renameSQL">
                                        <xsl:with-param name="SQLname" select="@Name"/>
                                        <xsl:with-param name="typeName" select="camed:dataType(@Type,@Decimal)"/>
                                        <xsl:with-param name="ns" select="if(string-length($prefixXMLns) &gt; 0) then concat($prefixXMLns,':') else ''"/>
                                    </xsl:call-template>
                                </xsl:variable>
                                
                                <!-- xsl:message> - <xsl:value-of select="$childName"/></xsl:message -->
                                
                                <xsl:element name="{$childName}" namespace="{$prefixXMLurl}">
                                        <xsl:if test="string-length($prefixXMLns) &gt; 0">
                                            <xsl:namespace name="{$prefixXMLns}"><xsl:value-of select="$prefixXMLurl"/></xsl:namespace>
                                        </xsl:if>
                                        <!-- xsl:attribute name="UID"><xsl:value-of select="if(string-length($prefixXMLns) &gt; 0) then $prefixXMLns else 'cam'"/>-<xsl:value-of select="generate-id()"/></xsl:attribute -->
                                    <!-- Now output the default type for item -->
                                    <xsl:choose>
                                        <xsl:when test="@Type = 'DATETIME'">%YYYY-MM-DD'T'HH:MI:SS.SSS%</xsl:when>
                                        <xsl:when test="@Type = 'DATE'">%YYYY-MM-DD%</xsl:when>
                                        <xsl:when test="@Type = 'BOOLEAN'">%boolean%</xsl:when>
                                        <xsl:when test="@Type = 'CHAR'">%token%</xsl:when>
                                        <xsl:when test="@Type = 'INT'">
                                            <xsl:choose>
                                                <xsl:when test="@Decimal = '0'">%123456%</xsl:when>
                                                <xsl:when test="@Decimal &gt; '2'">%12345.0000%</xsl:when>
                                                <xsl:otherwise>%1234.00%</xsl:otherwise>
                                            </xsl:choose>
                                        </xsl:when>
                                        <xsl:when test="@Type = 'DECIMAL'">%1234.00%</xsl:when>
                                        <xsl:otherwise>%string%</xsl:otherwise>
                                    </xsl:choose>
                                </xsl:element>             
                            </xsl:for-each>
                       
                    </xsl:element>
                </xsl:for-each>
            </xsl:element>
            </xsl:element>
            <as:BusinessUseContext>
                <as:Rules>
                    <as:default>
                        <as:context>
                            <xsl:for-each select="$SQLdefinitions//TableEntity">
                                <xsl:variable name="currentName" select="@Name"/>
                                <!-- xsl:variable name="thisName"><xsl:value-of select="concat($nsPrefix,$currentName)"/>details</xsl:variable -->
                                <xsl:variable name="thisName">
                                    <xsl:call-template name="renameSQL">
                                        <xsl:with-param name="SQLname" select="$currentName"/>
                                        <xsl:with-param name="typeName" select="string"/>
                                        <xsl:with-param name="ns" select="if(string-length($prefixXMLns) &gt; 0) then concat($prefixXMLns,':') else ''"/>
                                    </xsl:call-template>
                                </xsl:variable>
                                <as:constraint action="makeRepeatable(//{$thisName})" />
                                <xsl:for-each select="$SQLdefinitions//TableEntity/Column">
                                    <!-- xsl:variable name="childName" select="concat($nsPrefix,@Name)"/ -->
                                    <xsl:variable name="childName">
                                        <xsl:call-template name="renameSQL">
                                            <xsl:with-param name="SQLname" select="@Name"/>
                                            <xsl:with-param name="typeName" select="camed:dataType(@Type,@Decimal)"/>
                                            <xsl:with-param name="ns" select="if(string-length($prefixXMLns) &gt; 0) then concat($prefixXMLns,':') else ''"/>
                                        </xsl:call-template>
                                    </xsl:variable>
                                    <xsl:copy-of select="camed:setActionType(@Type,@Decimal,@Length,$thisName,$childName)"/>
                                    <xsl:if test="contains(@isNullable,'YES')"><as:constraint action="makeOptional(//{$thisName}/{$childName})" /></xsl:if>
                                    <xsl:if test="string-length(@Length) &gt; 0 and(not(contains(upper-case(@Type),'DATE'))) and(not(contains(upper-case(@Type),'TIME'))) and(not(contains(upper-case(@Type),'INT')))"><as:constraint action="setLength(//{$thisName}/{$childName},1-{@Length})" /></xsl:if>
                                </xsl:for-each>
                            </xsl:for-each>
                        </as:context>
                    </as:default>
                </as:Rules>
            </as:BusinessUseContext>
            <as:Extension name="uk.org.jcam.camed.extensions.StructureAnnotations">
                <xsl:variable name="DBname" select="if($SQLdefinitions//TableEntity/@Catalog) then $SQLdefinitions//TableEntity/@Catalog else $SQLdefinitions//TableEntity/@Schema"/>
                <xsl:for-each select="$SQLdefinitions//TableEntity">
                    <xsl:variable name="tableName" select="@Name"/>
                    <!-- xsl:variable name="thisName"><xsl:value-of select="concat($nsPrefix,$tableName)"/>details</xsl:variable -->
                    <xsl:variable name="thisName">
                        <xsl:call-template name="renameSQL">
                            <xsl:with-param name="SQLname" select="$tableName"/>
                            <xsl:with-param name="typeName" select="string"/>
                            <xsl:with-param name="ns" select="if(string-length($prefixXMLns) &gt; 0) then concat($prefixXMLns,':') else ''"/>
                        </xsl:call-template>
                    </xsl:variable>
                    <camed:annotation item="//{$thisName}">
                            <xsl:choose>
                                <xsl:when test="string-length(@Remarks) &gt; 0">
                                    <camed:documentation type="Definition"><xsl:value-of select="replace(@Remarks,'&#xA;','')"/></camed:documentation>
                                </xsl:when>
                                <xsl:otherwise/>
                            </xsl:choose>
                  </camed:annotation>
                  
                  <xsl:for-each select="$SQLdefinitions//TableEntity/Column">
                    <!-- xsl:variable name="childName" select="concat($nsPrefix,@Name)"/ -->
                      <xsl:variable name="childName">
                          <xsl:call-template name="renameSQL">
                              <xsl:with-param name="SQLname" select="@Name"/>
                              <xsl:with-param name="typeName" select="camed:dataType(@Type,@Decimal)"/>
                              <xsl:with-param name="ns" select="if(string-length($prefixXMLns) &gt; 0) then concat($prefixXMLns,':') else ''"/>
                          </xsl:call-template>
                      </xsl:variable>
                    <xsl:if test="string-length(@Remarks) &gt; 0">
                        <camed:annotation item="//{$thisName}/{$childName}">
                            <camed:documentation type="Definition"><xsl:value-of select="replace(@Remarks,'&#xA;','')"/></camed:documentation>
                        </camed:annotation>
                    </xsl:if>
                    <camed:annotation item="//{$thisName}/{$childName}">
                        <camed:documentation type="DBMapping"><xsl:value-of select="concat($DBname,'.',$tableName,'.',@Name)"/></camed:documentation>
                    </camed:annotation>
                    </xsl:for-each>
                </xsl:for-each>
            </as:Extension>
        </xsl:element>
    </xsl:template>
    
    <xsl:template name="renameSQL">
        <xsl:param name="SQLname"/>
        <xsl:param name="typeName"/>
        <xsl:param name="ns"/>
        
        <xsl:variable name="chkName" select="$SQLname"/>
        <xsl:variable name="thisName" select="$SQLname"/>
        <xsl:variable name="firstLetter" select="substring($chkName,1,1)"/>
        
        <xsl:variable name="newname">
            <xsl:choose>
                <xsl:when test="$renameMode='NIEM'">
                    
                    <xsl:variable name="adjustMixedName"><xsl:if test="string-length($typeName) &gt; 0 and(starts-with(upper-case($thisName),upper-case(normalize-space($typeName)))) and(not(upper-case($thisName) = $thisName))">true</xsl:if></xsl:variable>
                    <!-- xsl:message>This:  <xsl:value-of select="$thisName"/> Type: <xsl:value-of select="$typeName"/> - <xsl:value-of select="$adjustMixedName"/> - <xsl:value-of select="substring($thisName,1,string-length($typeName)) = $typeName"/> - <xsl:value-of select="string-length(normalize-space($typeName))"/></xsl:message -->
                    <xsl:choose>
                        <xsl:when test="$adjustMixedName='true'"><xsl:value-of select="concat($ns,camed:FIX-UPPER(upper-case($chkName),$renameMode,'ELEMENT'))"/></xsl:when>
                        <xsl:when test="camed:WordMatch($chkName,$renameMode)='true'"></xsl:when>
                        <xsl:when test="$firstLetter=$chkName and(upper-case($firstLetter)=$firstLetter)"></xsl:when><!-- skip one char upper case name -->
                        <xsl:when test="upper-case($chkName)=$chkName"><xsl:value-of select="concat($ns,camed:FIX-UPPER($chkName,$renameMode,'ELEMENT'))"/></xsl:when>                                      
                        <xsl:when test="not(upper-case($firstLetter)=$firstLetter)"><xsl:value-of select="concat($ns,replace(replace(concat(upper-case($firstLetter),substring-after($chkName,$firstLetter)),'_','-'),'\.','-'))"/></xsl:when><!-- Make first letter upper case -->
                        <xsl:when test="contains($chkName,'_') or(contains($chkName,'.')) or(contains($chkName,'-'))"><xsl:value-of select="concat($ns,replace(replace(replace($chkName,'-',''),'_',''),'\.','-'))"/></xsl:when><!-- Adjust special characters -->
                        <xsl:otherwise><xsl:value-of select="$chkName"/></xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:when test="$renameMode='CamelCase'">
                    <xsl:choose>
                        <xsl:when test="camed:WordMatch($chkName,$renameMode)='true'"/>
                        <xsl:when test="$firstLetter=$chkName and(upper-case($firstLetter)=$firstLetter)"/><!-- skip one char upper case name -->
                        <xsl:when test="upper-case($chkName)=$chkName"><xsl:value-of select="concat($ns,camed:FIX-UPPER($chkName,$renameMode,'ELEMENT'))"/></xsl:when>
                        <xsl:when test="not(upper-case($firstLetter)=$firstLetter)"><xsl:value-of select="concat($ns,upper-case($firstLetter),substring-after($chkName,$firstLetter))"/></xsl:when><!-- Make first letter upper case -->
                        <xsl:when test="contains($chkName,'_') or(contains($chkName,'.'))"><xsl:value-of select="concat($ns,replace(replace($chkName,'_',''),'\.','-'))"/></xsl:when><!-- Adjust special characters -->
                        <xsl:otherwise><xsl:value-of select="$chkName"/></xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:variable>
        <xsl:if test="string-length($newname) &gt; 0 and(not($newname=$ns))">                          
            <!-- Compute new element name -->
            <xsl:variable name="completeNewName">
                <xsl:choose>
                    <xsl:when test="$renameMode='NIEM'">
                        <!-- xsl:when test=""></xsl:when --><!-- ??? NIEM name property mode with property and mixed case -->
                        <!-- Now check NDR term rules on name and fix accordingly -->
                        <xsl:variable name="chkName" select="cam2func:has-representation-term($newname)"/>
                        <!-- xsl:message> - chkname - <xsl:value-of select="$chkName"/></xsl:message -->
                        <xsl:choose>
                            <xsl:when test="string-length($chkName) &gt; 0">
                                <!-- Now correct the representation term based on content type -->
                                <xsl:choose>
                                    <xsl:when test="not($source//Column[@Name=$thisName])"><xsl:value-of select="concat($newname,'Details')"/></xsl:when><!-- Parent elements -->
                                    <xsl:otherwise> 
                                        <xsl:variable name="parentName" select="$source//TableEntity/@Name"/>
                                        <!-- xsl:message> - <xsl:value-of select="$parentName"/> - <xsl:value-of select="$thisName"/></xsl:message -->
                                        <xsl:variable name="baseDataTypes">
                                            <xsl:for-each select="$source//Column[@Name=$thisName][1]">
                                                <!-- check the constraint details -->
                                                <xsl:variable name="constraint" select="camed:setActionType(@Type,@Decimal,@Length,$parentName[1],$thisName)"/>
                                                <!-- xsl:message> - <xsl:value-of select="$constraint/@action"/></xsl:message -->
                                                <xsl:value-of select="cam2func:rules2type($constraint/@action)"/>
                                            </xsl:for-each>
                                        </xsl:variable>
                                        <xsl:variable name="baseDataType" select="distinct-values(tokenize($baseDataTypes,'\|'))"/>
                                        <xsl:variable name="baseTypesCount" select="count($baseDataType)"/>
                                        <xsl:variable name="NameSuffix">
                                            <xsl:for-each select="$baseDataType">
                                                <xsl:if test="$baseTypesCount &lt; 3 or(not(.='string'))"><xsl:value-of select="cam2func:type2term(.)"/></xsl:if>
                                            </xsl:for-each>
                                        </xsl:variable>
                                        <xsl:choose>
                                            <xsl:when test="ends-with(upper-case($newname),upper-case($NameSuffix))">
                                                <xsl:value-of  select="concat(substring($newname,1,string-length($newname) - string-length($NameSuffix)),$NameSuffix)"/>
                                            </xsl:when>
                                            <xsl:otherwise><xsl:value-of select="concat($newname,$NameSuffix)"/></xsl:otherwise>
                                        </xsl:choose>
                                        <!-- xsl:message>Fix: <xsl:value-of select="$newname"/><xsl:value-of select="$NameSuffix"/> | <xsl:value-of select="$baseDataType"/></xsl:message -->
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>
                            <xsl:otherwise><xsl:value-of select="$newname"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise><xsl:value-of select="$newname"/></xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="newnameValue" select="if(contains($completeNewName,':')) then substring-after($completeNewName,':') else $completeNewName"/>
            <xsl:variable name="splitNewName"><xsl:value-of select="camed:splitNewWords($newnameValue,$newnameValue,'true')"/></xsl:variable>
            <xsl:variable name="spellChkName">
                <xsl:for-each select="tokenize($splitNewName,'\|')"><xsl:if test="not(.='')"><xsl:value-of  select="concat(.,' ')"/></xsl:if></xsl:for-each>
            </xsl:variable>
            <!-- Now write renaming details --> 
            <!-- xsl:value-of select="concat($completeNewName,' spellcheck=',$spellChkName,' origname=',$newname)"/ -->
            <xsl:value-of select="$completeNewName"/>
        </xsl:if>
    </xsl:template>

    <xsl:function name="camed:FIX-UPPER">
        <xsl:param name="name"/>
        <xsl:param name="mode"/>
        <xsl:param name="type"/>
        
        <xsl:variable name="lcName" select="lower-case($name)"/>
        
        <xsl:variable name="splitName">
            <xsl:value-of select="camed:splitWords($name,$lcName,'true')"/>
        </xsl:variable>
        <xsl:variable name="splitNameList" select="tokenize($splitName,'\|')"/>
        <xsl:variable name="revisedName">
            <xsl:for-each select="tokenize($splitName,'\|')">
                <xsl:variable name="nextWord" select="position() + 1"/>
                <xsl:value-of select="if(position()=1) then camed:WordDefaults(.,$splitNameList[$nextWord],$mode) else concat('|',camed:WordDefaults(.,$splitNameList[$nextWord],$mode))"/>
            </xsl:for-each>
        </xsl:variable>
        <xsl:variable name="newname"><xsl:value-of select="if($mode='NIEM') then replace($revisedName,'\|','') else replace($revisedName,'\|','-')"/></xsl:variable>
        <xsl:variable name="returnName"><xsl:value-of select="if($type='ELEMENT') then $newname else concat(lower-case(substring($newname,1,1)),substring($newname,2,string-length($newname) - 1))"/></xsl:variable>
        <xsl:value-of select="if($returnName=$name) then '' else $returnName"/>
    </xsl:function>
    
    <xsl:function name="camed:splitWords">
        <xsl:param name="words"/>
        <xsl:param name="lcWords"/>
        <xsl:param name="start"/>
        <xsl:if test="string-length($words) &gt; 0">
            <xsl:variable name="firstChar" select="substring($words,1,1)"/>
            <xsl:variable name="nextChars" select="substring-after($words,$firstChar)"/>
            <xsl:variable name="firstNextChar" select="substring($nextChars,1,1)"/>
            <xsl:choose>
                <xsl:when test="contains('.-_',$firstChar)">|<xsl:value-of select="camed:splitWords($nextChars,lower-case($nextChars),'true')"/></xsl:when>
                <xsl:when test="$start='true'"><xsl:value-of select="upper-case($firstChar)"/><xsl:value-of select="camed:splitWords($nextChars,lower-case($nextChars),'false')"/></xsl:when>
                <xsl:when test="$firstChar=substring($lcWords,1,1)"><xsl:value-of select="$firstChar"/><xsl:value-of select="camed:splitWords($nextChars,lower-case($nextChars),'false')"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="lower-case($firstChar)"/><xsl:value-of select="camed:splitWords($nextChars,lower-case($nextChars),'false')"/></xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:function>
    
    <xsl:function name="camed:dataType">
        <xsl:param name="typeName"/>
        <xsl:param name="decimal"/>
        <xsl:choose>
            <xsl:when test="$typeName = 'DATETIME'">DateTime</xsl:when>
            <xsl:when test="$typeName = 'DATE'">Date</xsl:when>
            <xsl:when test="$typeName = 'BOOLEAN'">Boolean</xsl:when>
            <xsl:when test="$typeName = 'CHAR'">Token</xsl:when>
            <xsl:when test="$typeName = 'INT'">
                <xsl:choose>
                    <xsl:when test="$decimal = '0'">Number</xsl:when>
                    <xsl:when test="$decimal &gt; '2'">Amount</xsl:when>
                    <xsl:otherwise>Value</xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="$typeName = 'DECIMAL'">Value</xsl:when>
            <xsl:otherwise/>
        </xsl:choose>
    </xsl:function>

    <xsl:function name="camed:setActionType">
        <xsl:param name="typeName"/>
        <xsl:param name="decimal"/>
        <xsl:param name="length"/>
        <xsl:param name="thisName"/>
        <xsl:param name="childName"/>
        <xsl:choose>
            <xsl:when test="$typeName = 'DATETIME'"><as:constraint action="setDateMask(//{$thisName}/{$childName},YYYY-MM-DD'T'HH:MI:SS.SSS)"/></xsl:when>
            <xsl:when test="$typeName = 'DATE'"><as:constraint action="setDateMask(//{$thisName}/{$childName},YYYY-MM-DD)"/></xsl:when>
            <xsl:when test="$typeName = 'BOOLEAN'"><as:constraint action="datatype(//{$thisName}/{$childName},boolean)"/></xsl:when>
            <xsl:when test="$typeName = 'CHAR'"><as:constraint action="datatype(//{$thisName}/{$childName},token)"/></xsl:when>
            <xsl:when test="$typeName = 'INT'">
                <xsl:variable name="lengthValue" select="if($length and(xsd:integer($length) &gt; 0)) then $length else 6"/>
                <xsl:choose>
                    <xsl:when test="$decimal = '0'"><as:constraint action="setNumberMask(//{$thisName}/{$childName},{substring($numbers,1,$lengthValue)})"/></xsl:when>
                    <xsl:when test="$decimal &gt; '2'"><as:constraint action="setNumberMask(//{$thisName}/{$childName},{substring($numbers,1,$lengthValue)}).{substring('##########',1,xsd:integer($decimal))})" /></xsl:when>
                    <xsl:otherwise><as:constraint action="setNumberMask(//{$thisName}/{$childName},{substring($numbers,1,$lengthValue - 2)}).##)" /></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="$typeName = 'DECIMAL'">
                <xsl:variable name="lengthValue" select="if($length and(xsd:integer($length) &gt; 2)) then $length else 6"/>
                <as:constraint action="setNumberMask(//{$thisName}/{$childName},{substring($numbers,1,$lengthValue - 2)}).##)" />
            </xsl:when>
            <xsl:otherwise/>
        </xsl:choose>
    </xsl:function>

</xsl:stylesheet>
