<?xml version="1.0" encoding="UTF-8"?> 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:lexs="http://usdoj.gov/leisp/lexs/3.1"
    xmlns:w="http://jcam.org.uk/editor/picklist/1.0"
    exclude-result-prefixes="xs xsd" version="2.0">

    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/ 

Author: David RR Webber, June, 2008 -->

    <!-- 

Purpose : Take a CAM wantlist and apply to CAM template format.  Insert rules accordingly.  Does not create duplicate rules.
Original : 06/13/2008 
Initials : drrw 
CAM ver : 1.1 
Description : Part of a set of tools to ease alignment of XSD based validations and documentation 
and CAM templates. 
How to use :  Provide picklist file as source input and CAM template file name to be processed out. The excludeTree, excludeElement and excludeAttribute rules inserted accordingly.
Updated - Inits - Change 
06/13/08 drrw Initial release.
09/14/12 drrw Changes to fix processing for latest CAM templates handling
-->
    <xsl:param name="CAMfileURL" select="'input.cam'"/>
    
    <xsl:output indent="yes" method="xml"/>
    
    <xsl:variable name="template" select="document($CAMfileURL)"/>
    
    <xsl:variable name="pickList" select="/"/>
    
    <xsl:variable name="version">1.01</xsl:variable>

    <xsl:template match="/">
        
        <xsl:message>Loading template from : <xsl:value-of select="$CAMfileURL"/></xsl:message>
        
        <xsl:message> - picklist <xsl:value-of select="$pickList//w:WantList/@w:product"/></xsl:message>
        <xsl:message> - excludes <xsl:value-of select="count($pickList//*[@w:isXclude = '1'])"/></xsl:message>
        
        <xsl:variable name="CAMexcludeRules">
            <rules
                xmlns:as="http://www.oasis-open.org/committees/cam"
                xmlns:camed="http://jcam.org.uk/editor"
                xmlns:lexs="http://usdoj.gov/leisp/lexs/3.1"
                xmlns:w="http://niem.gov/niem/wantlist/1">
            <xsl:apply-templates select="*"/>
            </rules>
        </xsl:variable>
        
        <xsl:element name="as:CAM">
        <xsl:copy-of select="$template/as:CAM/namespace::*"/>
        <xsl:copy-of select="$template/as:CAM/attribute::*"/>
        <xsl:copy-of select="$template/as:CAM/as:Header"/>
        <xsl:copy-of select="$template/as:CAM/as:AssemblyStructure"/>
        <xsl:call-template name="outputRules">
            <xsl:with-param name="newRules" select="$CAMexcludeRules"/>
        </xsl:call-template>
        <xsl:copy-of select="$template/as:CAM/as:include"/>
        <xsl:copy-of select="$template/as:CAM/as:Extension"/>
        </xsl:element>
        
    </xsl:template>
    
    <xsl:template name="outputRules">
        <xsl:param name="newRules"/>
        <as:BusinessUseContext>
            <as:Rules>
                <as:default>
                    <as:context>
                        <xsl:copy-of select="$template/as:CAM/as:BusinessUseContext/as:Rules/as:default/as:context/*"/>
                        <xsl:for-each select="$newRules/rules/as:constraint">
                            <xsl:variable name="temp" select="@action"></xsl:variable>
                            <xsl:if test="not(count($template/as:CAM/as:BusinessUseContext/as:Rules/as:default/as:context/as:constraint[@action=$temp]) &gt;0)">
                                <xsl:copy-of select="."></xsl:copy-of>
                            </xsl:if>
                        </xsl:for-each>
                    </as:context>
                </as:default>
                <xsl:copy-of select="$template/as:CAM/as:BusinessUseContext/as:Rules/as:context/."/>
            </as:Rules>
        </as:BusinessUseContext>
    </xsl:template>
    
   <xsl:template match="w:WantList">
       <xsl:message>Processing starting</xsl:message>
       <xsl:apply-templates select="Group"/>
       <xsl:message>Finished</xsl:message>
   </xsl:template> 
    
   <xsl:template match="Group">
       <!-- xsl:message>Group <xsl:value-of select="@w:n"/></xsl:message -->
       <xsl:choose>
       <xsl:when test="@w:isXclude='1'">
           <as:constraint action="excludeTree(//{replace(@w:n,'\\','/')})" />  
           <xsl:message> - exclude tree <xsl:value-of select="@w:n"/></xsl:message>
       </xsl:when>
       <xsl:otherwise><xsl:apply-templates select="Group|Element|Attribute"/></xsl:otherwise>
       </xsl:choose>
   </xsl:template>
    
    <xsl:template match="Attribute">
        <xsl:if test="@w:isXclude='1'">
            <as:constraint action="excludeAttribute(//{replace(@w:n,'\\','/')})" />  
            <xsl:message> - exclude attribute <xsl:value-of select="@w:n"/></xsl:message>
        </xsl:if>
    </xsl:template>

    <xsl:template match="Element">
        <xsl:choose>
        <xsl:when test="@w:isXclude='1'">
            <as:constraint action="excludeElement(//{replace(@w:n,'\\','/')})" /> 
            <xsl:message> - exclude element <xsl:value-of select="@w:n"/></xsl:message>
        </xsl:when>
        <xsl:otherwise><xsl:apply-templates select="Element|Attribute"/></xsl:otherwise>
       </xsl:choose>
    </xsl:template>

</xsl:stylesheet>